# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ShellLisp.py,v 1.7 2002/11/27 21:03:23 jalet Exp $
#
# $Log: ShellLisp.py,v $
# Revision 1.7  2002/11/27 21:03:23  jalet
# More variables added to the Lisp environment
#
# Revision 1.6  2002/11/20 23:58:00  jalet
# Much better setting of variables now.
# Wei He's work on ShellPHP integrated 'my way', but his Windows support
# will be added tomorrow (it's 1 a.m. here, and I must go to bed)
#
# Revision 1.5  2002/11/19 09:57:53  jalet
# Unneeded import
#
# Revision 1.4  2002/11/07 21:36:10  jalet
# Two new variables interned : argc and argv which respectively contain
# the number of "direct" arguments and their values (as a list).
#
# Revision 1.3  2002/11/07 15:53:54  jalet
# Forgot kw argument in ShellLisp.py
#
# Revision 1.2  2002/11/01 23:15:15  jalet
# Sources should be ok now.
#
# Revision 1.1.1.1  2002/11/01 22:51:26  jalet
# Initial import
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re, string
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM

from ZShellShell import Shell
from lisp import Lisper

manage_addShellLispForm = DTMLFile('dtml/ShellLispAdd', globals())

_marker = []  # Create a new marker object

def manage_addShellLisp(self, id, title="", REQUEST=None, submit=None):
    """Add a Lisp Shell to a folder.
    """
    id = str(id)
    self._setObject(id, ShellLisp(id, title))
    if REQUEST is not None:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
    
class ShellLisp(Shell) :
    """ShellLisp implementation"""
    
    meta_type='Shell (Lisp)'

    def Execute(self, script, bound_names, args, kw, iostreams):
        """Executes a Lisp Script."""
        (stdin, stdout, stderr) = iostreams
        lisper = Lisper(iostreams)
        
        # maybe find better names...
        lisper.intern("argc", len(args))
        lisper.intern("argv", args)
        
        # Force REQUEST to generate all items. Any better idea ?
        script.REQUEST.text()
        
        for (name, value) in bound_names.items() + kw.items() + script.REQUEST.items() :
            lisper.intern(self.Fix_VariableName(name), value)
            
        lisper.intern("environ", script.REQUEST.environ)
        lisper.intern("form", script.REQUEST.form)
        lisper.intern("cookies", script.REQUEST.cookies)
        
        s = script.REQUEST.SESSION
        lisper.intern("SESSION_ID", s.id)
        lisper.intern("SESSION_TOKEN", s.token)
        lisper.intern("SESSION", s._container)
        
        (username, password) = script.REQUEST._authUserPW()
        lisper.intern("username", username)
        lisper.intern("password", password)
        
        code = '\n'.join(script._body.split('\n')[1:])
        sexpr = lisper.r.get_sexpr(code)
        while sexpr:
            lisper.eval(sexpr)                # output catched here
            sexpr = lisper.r.get_sexpr()
        return stdout.getvalue()

InitializeClass(ShellLisp)
