# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ShellPHP.py,v 1.9 2002/11/27 20:31:54 jalet Exp $
#
# $Log: ShellPHP.py,v $
# Revision 1.9  2002/11/27 20:31:54  jalet
# Last changes before tarballing 0.3
#
# Revision 1.8  2002/11/21 13:59:37  jalet
# Session now decomposed in SESSION_ID, SESSION_TOKEN and SESSION (this last
# one is a dictionnary)
#
# Revision 1.7  2002/11/21 13:40:59  jalet
# QUERY_STRING reintroduced
#
# Revision 1.6  2002/11/21 13:28:52  jalet
# User credentials, SESSION are now passed somewhat correctly.
# ZOPE_VARS reintroduced in ShellPHP for compatibility with PHParser. Thanks
# to Wei He.
#
# Revision 1.5  2002/11/21 10:11:39  jalet
# Improved documentation
#
# Revision 1.4  2002/11/21 09:50:55  jalet
# More comments.
# php instead of php3
#
# Revision 1.3  2002/11/20 23:58:00  jalet
# Much better setting of variables now.
# Wei He's work on ShellPHP integrated 'my way', but his Windows support
# will be added tomorrow (it's 1 a.m. here, and I must go to bed)
#
# Revision 1.2  2002/11/19 13:26:25  jalet
# Code refactoring, now ShellUnix seems to be OK.
# Preliminary work on ShellPerl, seems to work perfectly fine !
#
# Revision 1.1  2002/11/07 21:55:05  jalet
# Forgotten main PHP skeleton
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM

from ZShellShell import Shell

manage_addShellPHPForm = DTMLFile('dtml/ShellPHPAdd', globals())

_marker = []  # Create a new marker object

def manage_addShellPHP(self, id, title="", REQUEST=None, submit=None):
    """Add a PHP Shell to a folder.
    """
    id = str(id)
    self._setObject(id, ShellPHP(id, title))
    if REQUEST is not None:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
    
class ShellPHP(Shell) :
    """ShellPHP implementation"""
    
    meta_type='Shell (PHP)'

    def Translate_Numeric(self, value) :
        return ('$', str(value))
        
    def Translate_Mapping(self, value) :
        strval = "array("
        for (k, v) in value.items() :
            strval += "'%s' => %s, \n\t" % (str(k), self.Translate_PythonValue(v)[1])
        if strval.endswith(", \n\t") :    
            strval = strval[:-4]
        strval += ")"
        return ('$', strval)
        
    def Translate_String(self, value) :     
        return ('$', "'%s'" % value.replace("'", "\\'").replace('\n', '\\\n'))
        
    def Translate_List(self, value) :    
        strval = "array("
        for v in value :
            (dummy, val) = self.Translate_PythonValue(v)
            strval += val + ", \n\t" 
        if strval.endswith(", \n\t") :    
            strval = strval[:-4]
        strval += ")"
        return ('$', strval)
    
    def Language_Assignment(self, name, value) :
        (vartype, value) = self.Translate_PythonValue(value)
        return '%s%s = %s;' % (vartype, self.Fix_VariableName(name), value)
        
    def Language_Prefix(self, shell) :    
        """Returns the language prefix to use."""
        return Shell.Language_Prefix(self, shell) + "\n<?php\n"
        
    def Language_Postfix(self, shell) :     
        """Returns the language postfix to use."""
        return Shell.Language_Postfix(self, shell) + "\n?>"
        
    def PostInitialization_Code(self) :    
        """Sets up a bunch of additional variables."""
        # initial code empty
        code = []
        
        # Construct $ZOPE_VARS from REQUEST.other 
        zopevars = {} 
        for item in self.REQUEST.other.keys():
            if not item in ('RESPONSE', 'PUBLISHED', 'PARENTS') :
                 zopevars.update({ item : self.REQUEST.other[item] })
        code.append(self.Language_Assignment("ZOPE_VARS", zopevars))
        
        # Construct $_POST from REQUEST.form only for POST requests
        request_method = self.REQUEST.environ.get('REQUEST_METHOD', 'GET')
        if request_method == 'POST' :
            code.append(self.Language_Assignment("HTTP_POST_VARS", self.REQUEST.form))
            code.append("$_POST = $HTTP_POST_VARS;")
        
        # Under Unix, ENV vars may appear as SERVER vars. Fix it
        code.append("if (@sizeof($_ENV) == 0) {$HTTP_ENV_VARS = $HTTP_SERVER_VARS; unset($HTTP_ENV_VARS['PHP_SELF']); $_ENV = $HTTP_ENV_VARS;} ")
 
        # Construct $_SERVER from REQUEST.environ
        code.append(self.Language_Assignment("HTTP_SERVER_VARS", self.REQUEST.environ))
        code.append(self.Language_Assignment("HTTP_SERVER_VARS['PHP_SELF']", "/" + self.absolute_url(1)))
        code.append("$_SERVER = $HTTP_SERVER_VARS;")

        # Construct $_COOKIE from REQUEST.cookies 
        code.append("$HTTP_COOKIE_VARS = $cookies;")
        code.append("$_COOKIE = $HTTP_COOKIE_VARS;")

        # TODO : the following lines don't seem to work here with php3 
        # Construct $_REQUEST according to variables_order
        # and register global vars if register_globals is set
        code.append("$__rgo = array('G'=>'_GET','P'=>'_POST','C'=>'_COOKIE','E'=>'_ENV','S'=>'_SERVER');")
        code.append("$__vo = ini_get('variables_order');")
        code.append("for ($__i = 0; $__i < 3; $__i++) {@$__m=&$$__rgo[$__vo[$__i]]; if (is_array($__m)) $_REQUEST += $__m;}")
        code.append("if (ini_get('register_globals')) {for ($__i = 0; $__i < 5; $__i++) {@$__m=&$$__rgo[$__vo[$__i]]; if (is_array($__m)) $GLOBALS += $__m;}}")
        code.append("unset($__m, $__rgo, $__i, $__vo);")
        
        # Construct $PHP_SELF
        code.append("$PHP_SELF = $HTTP_SERVER_VARS['PHP_SELF'];")
        code.append("")
        
        # TODO : It's bad to put QUERY_STRING into the environment so as to let PHP generate $_GET 
        # TODO : because the environment is shared AFAIK. Check consequences.
        os.putenv('QUERY_STRING', self.REQUEST.environ.get('QUERY_STRING', ''))
        
        # returns initialization code
        return code
        
    def Execute(self, script, bound_names, args, kw, iostreams):
        """Executes a PHP Script."""
        raw = self.CommonExecute("php", script, bound_names, args, kw, iostreams)
        PHPEOL = "\r?\n"
        (phpHeader,phpBody) = re.search("(?s)(?P<header>^.*?)%s(?P<body>.*)" % (PHPEOL*2),raw).groups() 
        for line in phpHeader.split(PHPEOL):
            if ':' in line:
                (name, value) = line.split(':', 1)
                if name.lower() == 'location' :
                    script.REQUEST.RESPONSE.redirect(value.strip())
                else:
                    script.REQUEST.RESPONSE.setHeader(name, value.strip())
        return phpBody

InitializeClass(ShellPHP)
