# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ShellPerl.py,v 1.6 2002/11/21 09:50:55 jalet Exp $
#
# $Log: ShellPerl.py,v $
# Revision 1.6  2002/11/21 09:50:55  jalet
# More comments.
# php instead of php3
#
# Revision 1.5  2002/11/20 23:58:00  jalet
# Much better setting of variables now.
# Wei He's work on ShellPHP integrated 'my way', but his Windows support
# will be added tomorrow (it's 1 a.m. here, and I must go to bed)
#
# Revision 1.4  2002/11/19 16:24:28  jalet
# ShellPerl now connects throguh xml-rpc only when needed, so Zope's concurrent
# threads number is not exhausted.
#
# Revision 1.3  2002/11/19 15:07:08  jalet
# Now Perl xml-rpc works !!!
#
# Revision 1.2  2002/11/19 13:26:25  jalet
# Code refactoring, now ShellUnix seems to be OK.
# Preliminary work on ShellPerl, seems to work perfectly fine !
#
# Revision 1.1  2002/11/19 09:58:17  jalet
# Perl interpreter skeleton added
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM

from ZShellShell import Shell

manage_addShellPerlForm = DTMLFile('dtml/ShellPerlAdd', globals())

_marker = []  # Create a new marker object

def manage_addShellPerl(self, id, title="", REQUEST=None, submit=None):
    """Add a Perl Shell to a folder.
    """
    id = str(id)
    self._setObject(id, ShellPerl(id, title))
    if REQUEST is not None:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
    
class ShellPerl(Shell) :
    """ShellPerl implementation"""
    
    meta_type='Shell (Perl)'

    def Translate_Numeric(self, value) :
        return ('$', str(value))
        
    def Translate_Mapping(self, value) :
        strval = "{"
        for (k, v) in value.items() :
            strval += "'%s' => %s, \n\t" % (str(k), self.Translate_PythonValue(v)[1])
        if strval.endswith(", \n\t") :    
            strval = strval[:-4]
        strval += "}"
        return ('$', strval)
        
    def Translate_String(self, value) :     
        return ('$', '"%s"' % value.replace('"', '\\"').replace('\n', '\\\n'))
        
    def Translate_List(self, value) :    
        strval = "["
        for v in value :
            (dummy, val) = self.Translate_PythonValue(v)
            strval += val + ", \n\t" 
        if strval.endswith(", \n\t") :    
            strval = strval[:-4]
        strval += "]"
        return ('@', strval)
    
    def Translate_Object(self, value) :
        try :
            value = value.absolute_url()
            return ('$', "ZShellScriptsClient->new(%s)" % self.Translate_PythonValue(value)[1])
        except AttributeError :    
            # we don't know what to do, so we convert to a string 
            return self.Translate_PythonValue(str(value))
        
    def Language_Assignment(self, name, value) :
        (vartype, value) = self.Translate_PythonValue(value)
        return 'my %s%s = %s;' % (vartype, self.Fix_VariableName(name), value)
        
    def PreInitialization_Code(self) :    
        """Sets up a bunch of additional variables."""
        code = """#
# And I thought I couldn't write Perl !
use Frontier::Client;
        
package ZShellScriptsClient;

  sub new {
      my $class=shift;
      my $self={};
      bless $self;
      if (defined $_[0]) {
          $self->{'url'} = shift;
      }
      return $self;
  }
  
  # does the same as a Python __getattr__()
  sub AUTOLOAD {
      my $self = shift;
      my $attr = $AUTOLOAD;
      $attr =~ s/.*:://;
      return unless $attr ne "DESTROY";
      # we connect each time to not exhaust Zope's number of concurrent threads
      my $server = Frontier::Client->new(url => $self->{url});
      return $server->call($attr, @_);
  }
"""
        return code.split('\n')
        
    def Execute(self, script, bound_names, args, kw, iostreams):
        """Executes a Perl Script."""
        return self.CommonExecute("perl", script, bound_names, args, kw, iostreams)
        
InitializeClass(ShellPerl)
