# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ShellPython.py,v 1.3 2002/11/07 21:42:14 jalet Exp $
#
# $Log: ShellPython.py,v $
# Revision 1.3  2002/11/07 21:42:14  jalet
# Security context should be set correctly for all type of scripts.
#
# Revision 1.2  2002/11/01 23:15:15  jalet
# Sources should be ok now.
#
# Revision 1.1.1.1  2002/11/01 22:51:25  jalet
# Initial import
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re, marshal
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM

from ZShellShell import Shell

# TODO : Track the Python bytecode version
import imp
Python_magic = imp.get_magic()
del imp

Script_magic = 3
_log_complaint = (
    'Some of your Scripts have stale code cached.  Since Zope cannot'
    ' use this code, startup will be slightly slower until these Scripts'
    ' are edited. You can automatically recompile all Scripts that have'
    ' this problem by visiting /manage_addProduct/PythonScripts/recompile'
    ' of your server in a browser.')


manage_addShellPythonForm = DTMLFile('dtml/ShellPythonAdd', globals())

_marker = []  # Create a new marker object

def manage_addShellPython(self, id, title="", REQUEST=None, submit=None):
    """Add a Python Shell to a folder.
    """
    id = str(id)
    self._setObject(id, ShellPython(id, title))
    if REQUEST is not None:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
    
class ShellPython(Shell) :
    """ShellPython implementation"""
    
    meta_type='Shell (Python)'

    def _newfun(self, script, code):
        g = {'__debug__': __debug__,
             '__builtins__': safe_builtins,
             '_getattr_': guarded_getattr,
             '_getitem_': guarded_getitem,
             '_write_': full_write_guard,
             '_print_': RestrictedPython.PrintCollector
             }
        l = {}
        exec code in g, l
        script._v_f = f = l.values()[0]
        return f

    def _compiler(self, *args):
        return RestrictedPython.compile_restricted_function(*args)
        
    def SaveState(self, script, state) :
        if (getattr(script, 'Python_magic', None) != Python_magic or
            getattr(script, 'Script_magic', None) != Script_magic):
            global _log_complaint
            if _log_complaint:
                LOG(script.meta_type, INFO, _log_complaint)    
                _log_complaint = 0
            # Changes here won't get saved, unless this Script is edited.
            body = script._body.rstrip()
            if body:
                script._body = body + '\n'
            self.Compile(script)
            script._v_change = 1
        elif script._code is None:
            script._v_f = None
        else:
            self._newfun(script, marshal.loads(script._code))

    def Compile(self, script):
        r = self._compiler(script._params, script._body or 'pass',
                           script.id, self.meta_type)
        code = r[0]
        errors = r[1]
        self.warnings = tuple(r[2])
        if errors:
            script._code = None
            script._v_f = None
            script._setFuncSignature((), (), 0)
            # Fix up syntax errors.
            filestring = '  File "<string>",'
            for i in range(len(errors)):
                line = errors[i]
                if line.startswith(filestring):
                    errors[i] = line.replace(filestring, '  Script', 1)
            script.errors = errors
            return

        script._code = marshal.dumps(code)
        script.errors = ()
        f = self._newfun(script, code)
        fc = f.func_code
        script._setFuncSignature(f.func_defaults, fc.co_varnames,
                               fc.co_argcount)
        script.Python_magic = Python_magic
        script.Script_magic = Script_magic
        script._v_change = 0
    
    def PrepareExecution(self, script, bound_names, args, kw) :
        """Prepares execution of a Python Script."""
        # TODO : check to see if it is useful here
        __traceback_info__ = bound_names, args, kw, script.func_defaults
        
        f = script._v_f
        if f is None:
            raise RuntimeError, '%s %s has errors.' % (script.meta_type, script.id)
        if bound_names is not None:
            # Updating func_globals directly *should* be thread-safe.
            f.func_globals.update(bound_names)
            
    def Execute(self, script, bound_names, args, kw, iostreams=(None, None, None)) :
        """Executes a Python Script."""
        
        # TODO : check to see if it is useful here
        __traceback_info__ = bound_names, args, kw, script.func_defaults
        
        f = script._v_f
        return apply(f, args, kw)

InitializeClass(ShellPython)
