# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ShellUnix.py,v 1.8 2002/11/20 23:58:00 jalet Exp $
#
# $Log: ShellUnix.py,v $
# Revision 1.8  2002/11/20 23:58:00  jalet
# Much better setting of variables now.
# Wei He's work on ShellPHP integrated 'my way', but his Windows support
# will be added tomorrow (it's 1 a.m. here, and I must go to bed)
#
# Revision 1.7  2002/11/19 16:24:28  jalet
# ShellPerl now connects throguh xml-rpc only when needed, so Zope's concurrent
# threads number is not exhausted.
#
# Revision 1.6  2002/11/19 13:26:25  jalet
# Code refactoring, now ShellUnix seems to be OK.
# Preliminary work on ShellPerl, seems to work perfectly fine !
#
# Revision 1.5  2002/11/19 11:29:52  jalet
# Now "Shell (Unix)" works correctly with non-named arguments too.
# Better output format as well. Read the source to see how to render
# it more "secure".
#
# Revision 1.4  2002/11/07 21:39:34  jalet
# Better now.
#
# Revision 1.3  2002/11/07 09:23:35  jalet
# kw argument's content is now added to the environment before launching
# the shell.
#
# Revision 1.2  2002/11/01 23:15:15  jalet
# Sources should be ok now.
#
# Revision 1.1.1.1  2002/11/01 22:51:25  jalet
# Initial import
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM

from ZShellShell import Shell

manage_addShellUnixForm = DTMLFile('dtml/ShellUnixAdd', globals())

_marker = []  # Create a new marker object

def manage_addShellUnix(self, id, title="", REQUEST=None, submit=None):
    """Add a Unix Shell to a folder.
    """
    id = str(id)
    self._setObject(id, ShellUnix(id, title))
    if REQUEST is not None:
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
    
class ShellUnix(Shell) :
    """ShellUnix implementation"""
    
    meta_type='Shell (Unix)'

    def Translate_Numeric(self, value) :
        return ('', str(value))
        
    def Translate_Mapping(self, value) :
        # TODO : do something, learn Bash maybe :-)
        # bash doesn't seem to implement associative arrays
        return self.Translate_PythonValue(str(value))
        
    def Translate_String(self, value) :     
        return ('', '"%s"' % value.replace('"', '\\"').replace('\n', '\\\n'))
        
    def Translate_List(self, value) :    
        # TODO : for now we will return it as a string
        strval = "("
        for v in value :
            (dummy, val) = self.Translate_PythonValue(v)
            strval += val + " " 
        if strval.endswith(" ") :    
            strval = strval[:-1]
        strval += ")"
        return ('', strval)
    
    def Language_Assignment(self, name, value) :
        return 'export %s=%s' % (self.Fix_VariableName(name), self.Translate_PythonValue(value)[1])
        
    def Execute(self, script, bound_names, args, kw, iostreams):
        """Executes a Unix Shell Script through the default shell **PROBABLY BASH ONLY**."""
        # 
        # default shell
        shell = "sh"
        
        # If you prefer to run a somewhat "restricted" shell then uncomment 
        # the following line :
        #shell = "rbash"
        #
        # alternatively we could create a chroot jail
        return self.CommonExecute(shell, script, bound_names, args, kw, iostreams)
        
InitializeClass(ShellUnix)
