# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ZShellScript.py,v 1.3 2002/11/07 21:42:14 jalet Exp $
#
# $Log: ZShellScript.py,v $
# Revision 1.3  2002/11/07 21:42:14  jalet
# Security context should be set correctly for all type of scripts.
#
# Revision 1.2  2002/11/01 23:15:15  jalet
# Sources should be ok now.
#
# Revision 1.1.1.1  2002/11/01 22:51:25  jalet
# Initial import
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re, posixpath
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM
from StringIO import StringIO

manage_addZShellScriptForm = DTMLFile('dtml/ZShellScriptAdd', globals())
_default_file = os.path.join(package_home(globals()),
                             'www', 'default_ZShellScript')

_marker = []  # Create a new marker object

def manage_addZShellScript(self, id, REQUEST=None, submit=None):
    """Add a ZShell script to a folder.
    """
    id = str(id)
    id = self._setObject(id, ZShellScript(id))
    if REQUEST is not None:
        file = REQUEST.form.get('file', '')
        if type(file) is not type(''): file = file.read()
        if not file:
            file = open(_default_file).read()
        self._getOb(id).write(file)
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
    
class ZShellScript(Script, Historical, Cacheable) :
    """ZShellScript implementation"""
    
    __implements__ = (WriteLockInterface,)
    
    meta_type='Script (ZShell)'
    
    _proxy_roles = ()

    _params = _body = ''
    errors = warnings = ()
    _v_change = 0

    manage_options = (
        {'label':'Edit',
         'action':'ZShellScriptHTML_editForm',
         'help': ('ZShellScripts', 'ZShellScript_edit.stx')},
        ) + BindingsUI.manage_options + (
        {'label':'Test',
         'action':'ZScriptHTML_tryForm',
         'help': ('ZShellScripts', 'ZShellScript_test.stx')},
        {'label':'Proxy',
         'action':'manage_proxyForm',
         'help': ('OFSP','DTML-DocumentOrMethod_Proxy.stx')},
        ) + Historical.manage_options + SimpleItem.manage_options + \
        Cacheable.manage_options

    def __init__(self, id):
        self.id = id
        self._zshell = None
        self.ZBindings_edit(defaultBindings)

    security = AccessControl.ClassSecurityInfo()

    security.declareObjectProtected('View')
    security.declareProtected('View', '__call__')
    security.declareProtected('View', 'get_IOStreams')

    security.declareProtected('View management screens',
      'ZShellScriptHTML_editForm', 'manage_main', 'read',
      'ZScriptHTML_tryForm', 'PrincipiaSearchSource',
      'document_src', 'params', 'body')

    ZShellScriptHTML_editForm = DTMLFile('dtml/ZShellScriptEdit', globals())
    manage = manage_main = ZShellScriptHTML_editForm
    ZShellScriptHTML_editForm._setName('ZShellScriptHTML_editForm')

    security.declareProtected('Change ZShell Scripts',
      'ZShellScriptHTML_editAction',
      'ZShellScript_setTitle', 'ZShellScript_edit',
      'ZShellScriptHTML_upload', 'ZShellScriptHTML_changePrefs')
    def ZShellScriptHTML_editAction(self, REQUEST, title, params, body):
        """Change the script's main parameters."""
        self.ZShellScript_setTitle(title)
        self.ZShellScript_edit(params, body)
        message = "Saved changes."
        return self.ZShellScriptHTML_editForm(self, REQUEST,
                                               manage_tabs_message=message)

    def ZShellScript_setTitle(self, title):
        title = str(title)
        if self.title != title:
            self.title = title
            self.ZCacheable_invalidate()

    def ZShellScript_edit(self, params, body):
        self._validateProxy()
        if self.wl_isLocked():
            raise ResourceLockedError, "The script is locked via WebDAV."
        if type(body) is not type(''):
            body = body.read()
        if self._params <> params or self._body <> body or self._v_change:
            self._params = str(params)
            self.write(body)

    def ZShellScriptHTML_upload(self, REQUEST, file=''):
        """Replace the body of the script with the text in file."""
        if self.wl_isLocked():
            raise ResourceLockedError, "The script is locked via WebDAV."
        if type(file) is not type(''): file = file.read()
        self.write(file)
        message = 'Saved changes.'
        return self.ZShellScriptHTML_editForm(self, REQUEST,
                                               manage_tabs_message=message)

    def ZShellScriptHTML_changePrefs(self, REQUEST, height=None, width=None,
                                      dtpref_cols='50', dtpref_rows='20'):
        """Change editing preferences."""
        szchh = {'Taller': 1, 'Shorter': -1, None: 0}
        szchw = {'Wider': 5, 'Narrower': -5, None: 0}
        try: rows = int(height)
        except: rows = max(1, int(dtpref_rows) + szchh.get(height, 0))
        try: cols = int(width)
        except: cols = max(40, int(dtpref_cols) + szchw.get(width, 0))
        e = (DateTime('GMT') + 365).rfc822()
        setc = REQUEST['RESPONSE'].setCookie
        setc('dtpref_rows', str(rows), path='/', expires=e)
        setc('dtpref_cols', str(cols), path='/', expires=e)
        REQUEST.form.update({'dtpref_cols': cols, 'dtpref_rows': rows})
        return apply(self.manage_main, (self, REQUEST), REQUEST.form)

    def ZScriptHTML_tryParams(self):
        """Parameters to test the script with."""
        param_names = []
        for name in self._params.split(','):
            name = name.strip()
            if name and name[0] != '*' and re.match('\w',name):
                param_names.append(name.split('=', 1)[0])
        return param_names

    def manage_historyCompare(self, rev1, rev2, REQUEST,
                              historyComparisonResults=''):
        return ZShellScript.inheritedAttribute('manage_historyCompare')(
            self, rev1, rev2, REQUEST,
            historyComparisonResults=html_diff(rev1.read(), rev2.read()) )

    def manage_haveProxy(self,r): return r in self._proxy_roles

    def _validateProxy(self, roles=None):
        if roles is None: roles = self._proxy_roles
        if not roles: return
        user = getSecurityManager().getUser()
        if user is not None and user.allowed(self, roles):
            return
        raise 'Forbidden', ('You are not authorized to change <em>%s</em> '
            'because you do not have proxy roles.\n<!--%s, %s-->' 
            % (self.id, user, roles))

    security.declareProtected('Change proxy roles',
      'manage_proxyForm', 'manage_proxy')

    manage_proxyForm = DTMLFile('dtml/ZShellScriptProxy', globals())
    def manage_proxy(self, roles=(), REQUEST=None):
        "Change Proxy Roles"
        self._validateProxy(roles)
        self._validateProxy()
        self.ZCacheable_invalidate()
        self._proxy_roles=tuple(roles)
        if REQUEST: return MessageDialog(
                    title  ='Success!',
                    message='Your changes have been saved',
                    action ='manage_main')

    security.declareProtected('Change ZShell Scripts',
      'PUT', 'manage_FTPput', 'write',
      'manage_historyCopy',
      'manage_beforeHistoryCopy', 'manage_afterHistoryCopy')

    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP PUT requests """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        self.write(REQUEST.get('BODY', ''))
        RESPONSE.setStatus(204)
        return RESPONSE        

    manage_FTPput = PUT

    def write(self, text):
        """ Change the Script by parsing a read()-style source text. """
        self._validateProxy()
        mdata = self._metadata_map()
        bindmap = self.getBindingAssignments().getAssignedNames()
        bup = 0

        st = 0
        try:
            while 1:
                # Find the next non-empty line
                m = _nonempty_line.search(text, st)
                if not m:
                    # There were no non-empty body lines
                    body = ''
                    break
                line = m.group(0).strip()
                if line[:2] != '##':
                    # We have found the first line of the body
                    body = text[m.start(0):]
                    break

                st = m.end(0)
                # Parse this header line
                if len(line) == 2 or line[2] == ' ' or '=' not in line:
                    # Null header line
                    continue
                k, v = line[2:].split('=', 1)
                k = k.strip().lower()
                v = v.strip()
                if not mdata.has_key(k):
                    SyntaxError, 'Unrecognized header line "%s"' % line
                if v == mdata[k]:
                    # Unchanged value
                    continue

                # Set metadata value
                if k == 'title':
                    self.title = v
                elif k == 'parameters':
                    self._params = v
                elif k[:5] == 'bind ':
                    bindmap[_nice_bind_names[k[5:]]] = v
                    bup = 1

            body = body.rstrip()
            if body:
                body = body + '\n'
            if body != self._body:
                self._body = body
            if bup:
                self._setupBindings(bindmap)

            self._makeFunction()
        except:
            LOG(self.meta_type, ERROR, 'write failed', error=sys.exc_info())
            raise

    def manage_FTPget(self):
        "Get source for FTP download"
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/plain')
        return self.read()

    def _metadata_map(self):
        m = {
            'title': self.title,
            'parameters': self._params,
           }
        bindmap = self.getBindingAssignments().getAssignedNames()
        for k, v in _nice_bind_names.items():
            m['bind '+k] = bindmap.get(v, '')
        return m

    def read(self):
        """ Generate a text representation of the Script source.

        Includes specially formatted comment lines for parameters,
        bindings, and the title.
        """
        # Construct metadata header lines, indented the same as the body.
        m = _first_indent.search(self._body)
        if m: prefix = m.group(0) + '##'
        else: prefix = '##'

        hlines = ['%s %s "%s"' % (prefix, self.meta_type, self.id)]
        mm = self._metadata_map().items()
        mm.sort()
        for kv in mm: 
            hlines.append('%s=%s' % kv)
        if self.errors:
            hlines.append('')
            hlines.append(' Errors:')
            for line in self.errors:
                hlines.append('  ' + line)
        if self.warnings:
            hlines.append('')
            hlines.append(' Warnings:')
            for line in self.warnings:
                hlines.append('  ' + line)
        hlines.append('')
        return ('\n' + prefix).join(hlines) + '\n' + self._body

    def params(self): return self._params
    def body(self): return self._body
    def get_size(self): return len(self.read())
    getSize = get_size

    def PrincipiaSearchSource(self):
        "Support for searching - the document's contents are searched."
        return "%s\n%s" % (self._params, self._body)

    def document_src(self, REQUEST=None, RESPONSE=None):
        """Return unprocessed document source."""
        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', 'text/plain')
        return self.read()

    def _editedBindings(self):
        if getattr(self, '_v_f', None) is not None:
            self._makeFunction()

    def _makeFunction(self, dummy=0):
        self.ZCacheable_invalidate()
        self._getZShell()
        if self._zshell is not None :
            self._zshell.Compile(self)
            
    def _getZShell(self) :    
        firstline = self._body.split('\n')[0]
        if firstline[:2] != "#!" :
            raise "A valid shell script must begin with #! ==> [%s]" % firstline, self._body
        self._zshell = self.toObject(firstline[2:].strip())

    def get_IOStreams(self) :
        """Returns the script's pseudo I/O streams content as a (in, out, err) tuple."""
        try :
            stdin = self.stdin.read()
        except AttributeError :    
            stdin = ""
        try :
            stdout = self.stdout.getvalue()
        except AttributeError :    
            stdout = ""
        try :
            stderr = self.stderr.getvalue()
        except AttributeError :    
            stderr = ""
        return (stdin, stdout, stderr)    
            
    def set_IOStreams(self, streams = None) :        
        self.stdout = StringIO()
        self.stderr = StringIO()
        if streams is None :
            self.stdin = StringIO("")
        else :
            (stdin, stdout, stderr) = streams
            self.stdin = StringIO(stdin)
            self.stdout.write(stdout)
            self.stderr.write(stderr)
    
    def __setstate__(self, state):
        Script.__setstate__(self, state)
        self._zshell.SaveState(self, state)
        
    def _getCachedResult(self, args, kw) :
        # Retrieve the value from the cache.
        keyset = result = None
        if self.ZCacheable_isCachingEnabled():
            # Prepare a cache key.
            keyset = kw.copy()
            asgns = self.getBindingAssignments()
            name_context = asgns.getAssignedName('name_context', None)
            if name_context:
                keyset[name_context] = self.aq_parent.getPhysicalPath()
            name_subpath = asgns.getAssignedName('name_subpath', None)
            if name_subpath:
                keyset[name_subpath] = self._getTraverseSubpath()
            # Note: perhaps we should cache based on name_ns also.
            keyset['*'] = args
            result = self.ZCacheable_get(keywords=keyset, default=_marker)
            if result is not _marker:
                # Got a cached value.
                (result, (stdin, stdout, stderr)) = result
                self.set_IOStreams((stdin, stdout, stderr))
        return (keyset, result)        
        
    def _exec(self, bound_names, args, kw):
        """Executes a ZShellScript."""
        self._getZShell()
        if self._zshell is not None :
            (keyset, result) = self._getCachedResult(args, kw)
            if keyset is not None :
                # Caching enabled
                if result is not _marker :
                    # Got a cached result
                    return result
                    
            self.set_IOStreams()
            
            # TODO : check to see if it is useful here
            __traceback_info__ = bound_names, args, kw, self.func_defaults
            self._zshell.PrepareExecution(self, bound_names, args, kw)
            
            # Execute the function in a new security context.
            security=getSecurityManager()
            security.addContext(self)
            try:
                result = self._zshell.Execute(self, bound_names, args, kw, (self.stdin, self.stdout, self.stderr))
            finally:
                security.removeContext(self)
            
            if keyset is not None:
                # Store the result in the cache.
                (stdin, stdout, stderr) = self.get_IOStreams()
                self.ZCacheable_set((result, (stdin, stdout, stderr)), keywords=keyset)
                
            return result    

    def toObject(self, path) :
            path = posixpath.normpath(path)
            try :
                    if path == '.' :
                            return self
                    elif path == '..' and hasattr(self,'aq_parent') :
                            if hasattr(self, "isTopLevelPrincipiaApplicationObject") and self.isTopLevelPrincipiaApplicationObject :
                                    return self
                            else :
                                    return self.aq_parent
                    else :
                            return self.unrestrictedTraverse(path)
            except IndexError:
                    # BUG in Zope 2.3.2 ! => doesn't take care correctly of the / folder
                    # when the path argument contains only '/'
                    return curdir.getPhysicalRoot()
            except AttributeError:
                    return None     # nothing, e.g. acl_users/*
            except KeyError:
                    return None     # empty folder it seems
            except TypeError:
                    return None     # What the hell is this ?

_first_indent = re.compile('(?m)^ *(?! |$)')
_nonempty_line = re.compile('(?m)^(.*\S.*)$')

_nice_bind_names = {'context': 'name_context', 'container': 'name_container',
                    'script': 'name_m_self', 'namespace': 'name_ns',
                    'subpath': 'name_subpath'}
                    
InitializeClass(ZShellScript)
