# ZShellScripts - (c) 2002 Jerome Alet
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# author: Jerome Alet - <alet@librelogiciel.com>
#
# $Id: ZShellShell.py,v 1.15 2002/11/21 13:59:37 jalet Exp $
#
# $Log: ZShellShell.py,v $
# Revision 1.15  2002/11/21 13:59:37  jalet
# Session now decomposed in SESSION_ID, SESSION_TOKEN and SESSION (this last
# one is a dictionnary)
#
# Revision 1.14  2002/11/21 13:28:52  jalet
# User credentials, SESSION are now passed somewhat correctly.
# ZOPE_VARS reintroduced in ShellPHP for compatibility with PHParser. Thanks
# to Wei He.
#
# Revision 1.13  2002/11/21 11:26:30  jalet
# Forgotten to transmit SESSION information...
#
# Revision 1.12  2002/11/21 09:50:55  jalet
# More comments.
# php instead of php3
#
# Revision 1.11  2002/11/20 23:58:00  jalet
# Much better setting of variables now.
# Wei He's work on ShellPHP integrated 'my way', but his Windows support
# will be added tomorrow (it's 1 a.m. here, and I must go to bed)
#
# Revision 1.10  2002/11/20 08:33:00  jalet
# Now language comments, prefix and postfix are parametrable, which allows
# greater language independance in ZShellShell.py
#
# Revision 1.9  2002/11/19 17:40:51  jalet
# Added support for win32 (untested)
#
# Revision 1.8  2002/11/19 16:24:58  jalet
# Again, forgotten unlink()
#
# Revision 1.7  2002/11/19 16:24:28  jalet
# ShellPerl now connects throguh xml-rpc only when needed, so Zope's concurrent
# threads number is not exhausted.
#
# Revision 1.6  2002/11/19 15:07:08  jalet
# Now Perl xml-rpc works !!!
#
# Revision 1.5  2002/11/19 13:52:08  jalet
# unlink() was not called (debugging), now it is.
#
# Revision 1.4  2002/11/19 13:26:25  jalet
# Code refactoring, now ShellUnix seems to be OK.
# Preliminary work on ShellPerl, seems to work perfectly fine !
#
# Revision 1.3  2002/11/07 21:42:14  jalet
# Security context should be set correctly for all type of scripts.
#
# Revision 1.2  2002/11/01 23:15:15  jalet
# Sources should be ok now.
#
# Revision 1.1.1.1  2002/11/01 22:51:25  jalet
# Initial import
#
#
#

#
# This software contains some code which license is as follows :
#
##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import sys, os, traceback, re, tempfile, popen2
from Globals import DTMLFile, MessageDialog, package_home, InitializeClass
import AccessControl, OFS, RestrictedPython
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from urllib import quote
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from Shared.DC.Scripts.Script import Script, BindingsUI, defaultBindings
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from AccessControl import full_write_guard, safe_builtins
from AccessControl.ZopeGuards import guarded_getattr, guarded_getitem
from zLOG import LOG, ERROR, INFO, PROBLEM

class DummyCode :    
    # We cheat !
    co_varnames = []
    co_argcount = 0
    
class Shell(SimpleItem) :
    """Shell implementation"""

    manage_options = (
        {'label':'Edit',
         'action':'ShellHTML_editForm',
         'help': ('Shell', 'Shell_edit.stx')},
        ) + BindingsUI.manage_options + SimpleItem.manage_options 

    def __init__(self, id, title=""):
        self.id = id
        self.title = title

    security = AccessControl.ClassSecurityInfo()

    security.declareObjectProtected('View')
    security.declareProtected('View', '__call__')

    security.declareProtected('View management screens',
      'ShellHTML_editForm', 'manage_main')

    ShellHTML_editForm = DTMLFile('dtml/ShellEdit', globals())
    manage = manage_main = ShellHTML_editForm
    ShellHTML_editForm._setName('ShellHTML_editForm')

    security.declareProtected('Change Shells', 'Shell_edit', 'ShellHTML_editAction')
    def ShellHTML_editAction(self, REQUEST, title):
        """Change the Shell's main parameters."""
        self.Shell_edit(title)
        message = "Saved changes."
        return self.ShellHTML_editForm(self, REQUEST,
                                               manage_tabs_message=message)

    def Shell_edit(self, title):
        """Edits Shell's properties."""
        self.title = title
        
    def Translate_Object(self, value) :
        """Default conversion for a Zope object is as its url, else as a string of text."""
        # get an url if it exists
        try :
            value = value.absolute_url()
        except AttributeError :    
            # we don't know what to do, so we convert to a string 
            value = str(value)
        return self.Translate_PythonValue(value)
        
    def Translate_PythonValue(self, value) :
        """Translate a Python value to another language value."""
        import types
        if type(value) in [types.TupleType, types.ListType] :
            return self.Translate_List(value)
        elif type(value) in [types.StringType, types.UnicodeType] :
            return self.Translate_String(value)
        elif type(value) == types.DictionaryType :
            return self.Translate_Mapping(value)
        elif type(value) in [types.IntType, types.FloatType, types.LongType] :
            return self.Translate_Numeric(value)
        else :
            return self.Translate_Object(value)
        
    def Language_Prefix(self, shell) :    
        """Returns the language prefix to use."""
        return "#! /usr/bin/env %s" % shell
        
    def Language_Postfix(self, shell) :     
        """Returns the language postfix to use."""
        return self.Language_Comment("End of %s script." % shell)
        
    def Language_Comment(self, comment) :    
        """Returns the message as an optionally multi-line comment in the language."""
        return "\n".join([("# %s" % line).strip() for line in comment.split('\n')])
        
    def Fix_VariableName(self, varname) :
        return varname.replace('-', '_').replace('.', '_')
        
    def PreInitialization_Code(self) :
        return []
        
    def PostInitialization_Code(self) :
        return []
        
    def CommonExecute(self, shell, script, bound_names, args, kw, iostreams) :
        """Execution method, common to all Unix scripts (Unix, Perl, PHP, etc...)."""
        #
        # first we set the shell environment
        firstlines = []
        firstlines.extend(self.Language_Prefix(shell).split('\n'))
        firstlines.append("")
        firstlines.append(self.Language_Comment("\nThis script was automatically generated by the ZShellScripts Zope Product."))
        firstlines.append(self.Language_Comment("It is licensed under the GNU General Public License\nof the Free Software Foundation."))
        firstlines.append(self.Language_Comment("\nInitialization code and environment settings\n"))
        
        # Sets up special pre-initialization code, if any
        firstlines.extend(self.PreInitialization_Code())
        
        # then really sets the variables    
        firstlines.append(self.Language_Comment("\nBound names and parameters"))
        for (name, value) in bound_names.items() + kw.items() :
            firstlines.append(self.Language_Assignment(name, value))
        firstlines.append("")    
        
        # Force REQUEST to generate all items. Any better idea ?
        script.REQUEST.text()
        
        firstlines.append(self.Language_Comment("\nREQUEST environment"))
        firstlines.append(self.Language_Assignment("environ", script.REQUEST.environ))
        firstlines.append("")    
        
        firstlines.append(self.Language_Comment("\nREQUEST form"))
        firstlines.append(self.Language_Assignment("form", script.REQUEST.form))
        firstlines.append("")    
        
        firstlines.append(self.Language_Comment("\nREQUEST cookies"))
        firstlines.append(self.Language_Assignment("cookies", script.REQUEST.cookies))
        firstlines.append("")    
        
        firstlines.append(self.Language_Comment("\nREQUEST SESSION"))
        s = script.REQUEST.SESSION
        firstlines.append(self.Language_Assignment("SESSION_ID", s.id))
        firstlines.append(self.Language_Assignment("SESSION_TOKEN", s.token))
        firstlines.append(self.Language_Assignment("SESSION", s._container))
        firstlines.append("")    
        
        firstlines.append(self.Language_Comment("\nRest of REQUEST items, we keep duplicates like in Python"))
        for (name, value) in [(name, value) for (name, value) in script.REQUEST.items() if name not in ["SESSION"] ] :
            firstlines.append(self.Language_Assignment(name, value))
        firstlines.append("")    
        
        firstlines.append(self.Language_Comment("\nUser credentials"))
        (username, password) = script.REQUEST._authUserPW()
        firstlines.append(self.Language_Assignment("username", username))
        firstlines.append(self.Language_Assignment("password", password))
        firstlines.append("")    
        
        # Sets up special post-initialization code, if any
        firstlines.extend(self.PostInitialization_Code())
            
        # then we add the user's code
        firstlines.append(self.Language_Comment("User code is below :"))    
        firstlines.extend(script._body.split('\n')[1:])
        firstlines.extend(self.Language_Postfix(shell).split('\n'))
        
        # we create the shell script on disk
        temp = tempfile.mktemp()
        ftemp = open(temp, "w")
        ftemp.write('\n'.join(firstlines))
        ftemp.close()
        
        # assign pseudo I/O streams
        (stdin, stdout, stderr) = iostreams
        
        # compute the command line to launch
        commandline = "%s %s" % (temp, " ".join(['"%s"' % self.Translate_PythonValue(arg)[1] for arg in args]))
        
        # WARNING : we render the script executable
        os.chmod(temp, 0700)
        
        # launch it
        # For any system, the interpreter must be in the PATH
        if sys.platform == "win32" :
            class Process :
                pass
            process = Process()    
            (process.tochild, process.fromchild, process.childerr) = os.popen3("%s" % commandline, "b")
        else :    
            process = popen2.Popen3("%s" % commandline, capturestderr=1)
        process.tochild.write(stdin.read())
        process.tochild.close()
        if sys.platform == "win32" :
            retcode = 0 # unknown
        else :    
            retcode = process.wait()
        
        # Now delete the script immediately
        os.unlink(temp)
        
        out = process.fromchild.read()
        err = process.childerr.read()
        stdout.write(out)
        stderr.write(err) # should we raise an exception if not empty ?
        #
        # TODO : what to do with retcode ?
        # TODO : check if this is the best behavior :
        return "\n".join([out, err])
        
    def SaveState(self, script, state) :
        """Dummy method. You can override."""
        pass

    def PrepareExecution(self, script, bound_names, args, kw) :
        """Dummy method. You can override."""
        pass
        
    def Compile(self, script):
        """Dummy method. You can override."""
        script.func_code = DummyCode()

