#! lisp
(def foo
  (lambda (x y)
    (* x y)))

(print (foo 12 55))

(def square (lambda (x) (* x x)))

(def map
  (lambda (fn args)
    (if (null? args)
        '()
        (cons (fn (first args))
              (map fn (rest args))))))

(print (map square '(2 4 5 7)))

(def member 
  (lambda (elt lst)
    (cond 
     ((null? lst) *false*)
     ((== elt (first lst)) elt)
     (*true* (member elt (rest lst))))))

(print (member 'c '(a b c d)))
(print (member 'd '(a b c d)))
(print (member 'e '(a b c d)))

(def defun
  (macro (name args body)
    `(def ,name (lambda ,args ,body))))

(def define
  (macro (head tail)
    (if (list? head)
        `(def ,(car head) (lambda ,(rest head) (begin ,tail)))
      `(def ,head ,tail))))

(print (macro-expand (define foo 12)))
(print (macro-expand (define (square x) (* x x))))

(def list
  (lambda (&rest l) l))

(def eval
  (macro (code)
    `(begin ,@code)))
    
(def apply
  (macro (fn args)
    `(,fn ,@args)))

(def incf
  (macro (x)
    `(setq ,x (+ ,x 1))))

(def bob 33)
(print 'bob bob)
(apply incf (bob))
(print 'bob bob)
(print (apply square (bob)))

(def make-value
  (lambda (value)
    (let ((dispatch
           (lambda (message)
             (cond ((== message 'get)
                    (lambda () value))
                   ((== message 'set)
                    (lambda (new-value)
                      (setq value new-value)))
                   (*true* 'unknown-message!)))))
      dispatch)))

(def bob (make-value 14))
((bob 'get))
((bob 'set) 42)
((bob 'get))

(def fib
  (lambda (x)
    (if (< x 2)
        x
        (+ (fib (- x 1)) (fib (- x 2))))))

(print "doing (fib 15) - this will be slow...")
(print "The answer should be 610. It is:" (fib 15))
